@@ -7,6 +7,7 @@ import logging
 import re
 import signal
 import sys
+from base64 import b64encode
 from gettext import gettext as _
 
 from astersay.exceptions import (
@@ -24,8 +25,7 @@ def quote_value(string):
     if not isinstance(string, str):
         string = str(string)
     if '"' in string:
-        raise ValueError(
-            _('Строка %r содержит двойную кавычку.') % string)
+        string = b64encode(string.encode()).decode()
     return '"%s"' % string
 
 
