@@ -8,7 +8,6 @@ from logging import getLogger
 
 from astersay.agi import Agi
 from astersay.conf import settings
-from astersay.dialog import Dialog
 
 
 logger = getLogger(__name__)
@@ -50,6 +49,8 @@ class App:
         assert self.agi
         assert self.settings
         assert self.stream
+
+        Dialog = self.settings.get_dialog_class()
         dialog = Dialog(self.agi, self.settings, self.stream)
         dialog.run()
 
