@@ -5,6 +5,7 @@
 #
 import grpc
 import threading
+from copy import deepcopy
 from gettext import gettext as _
 
 from astersay.backends.base import Listener
@@ -33,14 +34,16 @@ class YandexRecognizer(YandexBaseBackend):
     # Счётчик текущих предварительных результатов. Сбрасывается финальным.
     partials_counter = 0
 
-    def __init__(self, settings):
+    def __init__(self, settings, extra=None):
         YandexBaseBackend.__init__(self)
         self.set_token_manager(settings)
 
         conf = settings.yandex
         self.folder_id = conf['auth']['folder_id']
 
-        recognizer = conf['recognizer']
+        recognizer = deepcopy(conf['recognizer'])
+        if extra:
+            recognizer.update(extra)
 
         language = recognizer['language_code']
         assert language in self.LANGUAGES
