@@ -669,13 +669,16 @@ class Settings:
         }
         filename = export.format(**kw)
         makedirs(dirname(filename), 0o700, exist_ok=True)
-        data = make_export_data(agi_params_data, export_data, text_buffers, **extra)
+        data = make_export_data(
+            agi_params_data,
+            export_data,
+            text_buffers,
+            dialog=self.dialog_name,
+            workdir=self.work_dir,
+            **extra)
         json.dump(data, open(filename, 'w'),
                   indent=4, ensure_ascii=False)
-        export_thread = Thread(target=self.remote_export_dialog,
-                               args=(filename, data))
-        export_thread.start()
-
+        self.remote_export_dialog(filename, data)
         return filename
 
     def remote_export_dialog(self, filename, data=None):
@@ -683,39 +686,83 @@ class Settings:
         if not export.get('active'):
             return
         assert export['mode'] in ('file', 'data')
-        assert export['retries_on_failure'] > 0
+        assert export['retries_on_failure'] >= 0
 
         logger_info = logger.info
         logger_error = logger.error
         request_url = export['url']
-        request_args = {'headers': export['headers']}
+        request_kwargs = {
+            'headers': export.get('headers', {}),
+            'timeout': export.get('timeout', (2, 5)),
+        }
 
         if export['mode'] == 'file':
-            request_args['files'] = {filename: open(filename, 'rb')}
-            request_args['data'] = {'filename': basename(filename)}
-        if export['mode'] == 'data':
+            request_kwargs['files'] = {'file': (
+                basename(filename),
+                open(filename, 'rb'),
+                'application/json',
+                {},
+            )}
+        else:
+            # В любом другом случае мы всё равно должны отправить данные.
             if not data:
                 data = json.load(open(filename))
-            request_args['data'] = data
-            request_args['params'] = {'filename': basename(filename)}
+            request_kwargs['json'] = data
 
-        for n in [export['retries_on_failure']]:
-            try:
-                res = requests.post(request_url, timeout=(2, 5), **request_args)
-                if res.status_code == 200:
-                    logger_info(_('Диалог экспортирован на удаленный сервер %(url)s в режиме %(mode)s'),
-                                {'url': request_url, 'mode': export['mode']})
+        def send(attempts):
+            for n in range(attempts):
+                try:
+                    res = requests.post(request_url, **request_kwargs)
+                    try:
+                        data = res.json()
+                    except Exception:
+                        data = res.text
+                    assert res.status_code == 200, _(
+                        'Не удалось экспортировать диалог на удалённый сервис. '
+                        'Сервер ответил кодом %(code)r: %(data)s'
+                    ) % {
+                        'code': res.status_code,
+                        'data': data,
+                    }
+                except AssertionError as e:
+                    logger_error(str(e))
+                except requests.exceptions.Timeout:
+                    logger_error(
+                        _('Timeout при экспорте диалога на удаленный сервер.'))
+                except requests.exceptions.RequestException as e:
+                    logger_error(
+                        _('Ошибка при экспорте диалога на удаленный сервер: %s'),
+                        e, exc_info=e)
+                else:
+                    logger_info(
+                        _('Диалог экспортирован на удаленный сервер %(url)s '
+                          'в режиме %(mode)s'),
+                        {'url': request_url, 'mode': export['mode']})
                     if export['delete_local_on_success'] and filename:
                         try:
                             remove(filename)
-                            logger_info(_('Локальный файл диалога %s удален'), filename)
                         except FileNotFoundError:
-                            logger_error(_('Локальный файл диалога %s не найден.'), filename)
-                    break
-            except requests.exceptions.Timeout:
-                logger_error(_('Timeout при экспорте диалога на удаленный сервер.'))
-            except requests.exceptions.RequestException as e:
-                logger_error(_('Ошибка при экспорте диалога на удаленный сервер: %s'), e.strerror, exc_info=e)
+                            logger_error(
+                                _('Локальный файл диалога %s не найден.'),
+                                filename)
+                        else:
+                            logger_info(_('Локальный файл диалога %s удален'),
+                                        filename)
+                    return True
+            return False
+
+        # Первая попытка выполняется в текущем контексте, чтобы можно было
+        # гарантировать успешность доставки. Это бывает необходимо для
+        # продолжения каких-либо действий в астериске.
+        # Если первая попытка была неуспешной, то остальные выполняются в
+        # отдельном потоке.
+
+        success = send(1)
+        retries = export['retries_on_failure']
+
+        if not success and retries:
+            t = Thread(target=send, args=(retries,))
+            t.start()
 
     def get_stop_file(self):
         if self.main.get('disable_stop_voices'):
