@@ -158,6 +158,8 @@ class BaseDialog:
     _silence_from = None
     # Флаг, при установке которого сбрасывается счётчик секунд молчания.
     _reset_silence = False
+    # Список истории действий внутри блока (скрипта).
+    _blocklog = None
 
     def __init__(self, agi, settings, stream):
         self.agi = agi
@@ -307,7 +309,7 @@ class BaseDialog:
         return result, text
 
     def say(self, text: str, voice: str, escape_digits: str, nonblocking=False,
-            reset_silense=True):
+            reset_silense=True, disable_blocklog=False):
         if not text and not voice:
             log_error(_('Текст и файл голоса отсутствуют.'))
             return
@@ -372,6 +374,11 @@ class BaseDialog:
         thread = Thread(target=runner, daemon=True)
         thread.start()
 
+        if not disable_blocklog:
+            self.blocklog(
+                'robot', text=text, voice=voice, nonblocking=nonblocking,
+                escape_digits=escape_digits, reset_silense=reset_silense)
+
         self._voice_nonblocking = nonblocking
         if not nonblocking:
             log_debug(_('Останавливаем основной поток на %s секунд.'), duration)
@@ -535,8 +542,11 @@ class BaseDialog:
             for key, value in self.export.items():
                 if key.startswith('_'):
                     continue
+                # В Asterisk отправляем только самые примитивные типы.
+                if value is not None and not isinstance(value, (str, int, float, bool)):
+                    continue
                 value = str(value)
-                log_debug(_('Экспорт %(key)s=%(value)s'), {
+                log_debug(_('Передача в Asterisk %(key)s=%(value)s'), {
                     'key': key, 'value': value})
                 agi.set(key, value)
         except AgiHangupError:
@@ -582,7 +592,7 @@ class BaseDialog:
             'variables': variables,
             'chain': self.chain,
             'text_buffers': self.text_buffers,
-            'config': config
+            'config': config or {},
         }
         if result is None or \
                 isinstance(result, (str, bool, list, tuple, int, float)):
@@ -650,6 +660,13 @@ class BaseDialog:
         tempfile.close()
         return res, plugin_data
 
+    def blocklog(self, logname, *args, **kwargs):
+        log = self._blocklog
+        if log is None:
+            self._blocklog = log = []
+        now = datetime.utcnow()
+        log.append(['%sZ' % now.isoformat(), logname, args, kwargs])
+
     def main_script(self):
         settings = self.settings
         scheme = settings.dialog_scheme
@@ -675,6 +692,9 @@ class BaseDialog:
             # Здесь должна возникать ошибка AttributeError для сломанных схем.
             method = getattr(self, '_script_%s' % scrypt_type)
 
+            # Сбрасываем логгирование для блока (скрипта).
+            self._blocklog = None
+
             debug_msg = '  SCRIPT %s'
             debug_args = [name.upper()]
             if 'group' in process:
@@ -716,6 +736,8 @@ class BaseDialog:
             if plugin_name:
                 plugin_conf = process.get('plugin_conf')
                 self.execute_plugin(plugin_name, result, speech, variables, plugin_conf)
+                self.blocklog('plugin', name=plugin_name, conf=plugin_conf,
+                              result=result, speech=speech)
 
             # Устанавливаем следующий процесс.
             success = process.get('success')
@@ -770,7 +792,7 @@ class BaseDialog:
             # Восстанавливаем константы. Это проще, чем городить лес кода.
             variables.update(constants)
             # Сохраняем цепочку вызовов скриптов.
-            chain.append([name, result, speech])
+            chain.append([name, result, speech, self._blocklog])
 
         log_debug(_('Главный скрипт завершён.'))
 
@@ -906,6 +928,7 @@ class BaseDialog:
         if speech:
             answer = speech
             log_debug(_('Ответ: %r'), answer)
+            self.blocklog('answer', answer=answer, speech=speech)
             prepare_answer(
                 answer=answer, source=answer, export_name=export_name,
                 script=script, variables=variables)
@@ -916,7 +939,11 @@ class BaseDialog:
             # перед say_success.
             plugin_name = script.get('plugin')
             if plugin_name:
-                self.execute_plugin(plugin_name, answer, speech, variables)
+                plugin_conf = script.get('plugin_conf')
+                self.execute_plugin(plugin_name, answer, speech, variables,
+                                    plugin_conf)
+                self.blocklog('plugin', name=plugin_name, conf=plugin_conf,
+                              result=answer, speech=speech)
 
             saydata = script.get('say_success')
             if saydata:
@@ -927,6 +954,8 @@ class BaseDialog:
                 self._agi_execute(commands, 'after_say_success')
         else:
             answer = None
+            if speech:
+                self.blocklog('answer', speech=speech, invalid=True)
             log_debug(_('Ответа нет.'))
             saydata = script.get('say_fail')
             if saydata:
@@ -1013,6 +1042,7 @@ class BaseDialog:
             if names:
                 length_names = int(script.get('length_names', 2))
                 answer = ' '.join(names[:length_names])
+                self.blocklog('answer', answer=answer, speech=speech)
                 log_debug(_('Ответ: %r'), answer)
                 prepare_answer(
                     answer=answer, source=answer, export_name=export_name,
@@ -1030,6 +1060,8 @@ class BaseDialog:
                 break
             elif attempt < attempts_count - 1:
                 if attempts.get('new_buffer', False):
+                    if speech:
+                        self.blocklog('answer', speech=speech, invalid=True)
                     self.make_text_buffer()
                 saydata = attempts.get('say')
                 if saydata:
@@ -1042,6 +1074,8 @@ class BaseDialog:
         self.stop_recognizing()
 
         if not names:
+            if speech:
+                self.blocklog('answer', speech=speech, invalid=True)
             log_debug(_('Ответа нет.'))
             saydata = script.get('say_fail')
             if saydata:
@@ -1136,6 +1170,7 @@ class BaseDialog:
                             say_silence, say_silence_index, variables)
             if match:
                 answer = match.group()
+                self.blocklog('answer', answer=answer, speech=speech)
                 log_debug(_('Ответ: %r'), answer)
                 prepare_answer(
                     answer=answer, source=answer, export_name=export_name,
@@ -1153,6 +1188,8 @@ class BaseDialog:
                 break
             elif attempt < attempts_count - 1:
                 if attempts.get('new_buffer', False):
+                    if speech:
+                        self.blocklog('answer', speech=speech, invalid=True)
                     self.make_text_buffer()
                 saydata = attempts.get('say')
                 if saydata:
@@ -1165,6 +1202,8 @@ class BaseDialog:
         self.stop_recognizing()
 
         if not match:
+            if speech:
+                self.blocklog('answer', speech=speech, invalid=True)
             log_debug(_('Ответа нет.'))
             saydata = script.get('say_fail')
             if saydata:
@@ -1258,6 +1297,7 @@ class BaseDialog:
                             say_silence, say_silence_index, variables)
             if response:
                 answer = response
+                self.blocklog('answer', answer=answer, speech=speech)
                 log_debug(_('Ответ: %r'), answer)
                 prepare_answer(
                     answer=answer, source=speech, export_name=export_name,
@@ -1275,6 +1315,8 @@ class BaseDialog:
                 break
             elif attempt < attempts_count - 1:
                 if attempts.get('new_buffer', False):
+                    if speech:
+                        self.blocklog('answer', speech=speech, invalid=True)
                     self.make_text_buffer()
                 saydata = attempts.get('say')
                 if saydata:
@@ -1287,6 +1329,8 @@ class BaseDialog:
         self.stop_recognizing()
 
         if not response:
+            if speech:
+                self.blocklog('answer', speech=speech, invalid=True)
             log_debug(_('Ответа нет.'))
             saydata = script.get('say_fail')
             if saydata:
@@ -1328,7 +1372,10 @@ class BaseDialog:
             plugin_conf = script.get('plugin_before_conf')
             res, plugin_data = self.execute_plugin(
                 plugin_name, result, '', variables, plugin_conf)
+            self.blocklog('plugin', name=plugin_name, conf=plugin_conf,
+                          result=result, speech='')
             result = plugin_data.get('result')
+            self.blocklog('plugin_return', result)
 
         saydata = script.get('say_start')
         if saydata:
@@ -1343,7 +1390,10 @@ class BaseDialog:
             plugin_conf = script.get('plugin_after_conf')
             res, plugin_data = self.execute_plugin(
                 plugin_name, result, '', variables, plugin_conf)
+            self.blocklog('plugin', name=plugin_name, conf=plugin_conf,
+                          result=result, speech='')
             result = plugin_data.get('result')
+            self.blocklog('plugin_return', result)
 
         if result:
             log_debug(_('Результат есть.'))
@@ -1368,4 +1418,6 @@ class BaseDialog:
         self._agi_execute(commands, 'after')
 
         speech = self.text_from_text_buffer()
+        if speech:
+            self.blocklog('answer', speech=speech)
         return result, speech
